/*
** HOOK.C
** 
** Copyright (c) 1989 by Digital Equipment Corporation
**                    All rights reserved.
**
** This module contains a system hook procedure and some supporting
** functions for it.  Note that hooks on the system queue need to
** be in DLLs so that their code is guaranteed to be accessible.
*/

/*
** Includes
*/
#define INCL_PM
#define INCL_DOS
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "keymap.h"
#include "compose.h"

#define DUMP(f,x)\
{ FILE *fout;\
  fout = fopen("C:\\HOOK.OUT", "a");\
  fprintf(fout, #x ## " = " ## f ## "\n", x);\
  fclose(fout);\
}

/*
** Global variables
*/
int         iCodePage=KMP_SETCP_QUERY;

/*
** Function prototypes.
*/
unsigned int iso2cp(unsigned int cin);

/*
** This is what we use for a virtual key id in our virtual key table
** so that we know if a key is being used as the compose key.  Note
** that it's at the upper end of allowable values, but not at the very
** end, we use 255 (-1) to indicate that no virtual key was selected
** for this scancode.  See MAP.TXT
*/
#define VK_LEAVEALONE 255
#define VK_COMPOSE 254

/*
** static variables
*/
static HWND hwndCompose;
static UCHAR map_sc[256];
static UCHAR map_vk[256];



/*
** get_hex_dec_oct
**
** Support function for reading the INI file.
*/
unsigned int get_hex_dec_oct(char *str)
{
   char          *p;
   unsigned int  u;

   p = strchr(str, 'x');
   if (!p)
      p = strchr(str, 'X');
   if (p)
   {
      sscanf(p+1, "%x", &u);
   }
   else
   {
      p = strchr(str, '\\');
      if (p)
      {
         sscanf(p+1, "%o", &u);
      }
      else
      {
         sscanf(str, "%u", &u);
      }
   }

   return u;
}



/*
** TranslateVKey
**
** During the reading of the scancode mapping file, we'll need to parse
** the string that indicates the virtual key assignment for a given
** scancode.  The following code is made by running an editing macro
** on a chunk of PMWIN.H (from the SDK, SoftSet, or whatever).  This is
** brute force...
*/
UCHAR TranslateVKey(CHAR *vkeystr)
{
   if (!strcmp("VK_BUTTON1", vkeystr))
      return VK_BUTTON1;
   else if (!strcmp("VK_BUTTON2", vkeystr))
      return VK_BUTTON2;
   else if (!strcmp("VK_BUTTON3", vkeystr))
      return VK_BUTTON3;
   else if (!strcmp("VK_BREAK", vkeystr))
      return VK_BREAK;
   else if (!strcmp("VK_BACKSPACE", vkeystr))
      return VK_BACKSPACE;
   else if (!strcmp("VK_TAB", vkeystr))
      return VK_TAB;
   else if (!strcmp("VK_BACKTAB", vkeystr))
      return VK_BACKTAB;
   else if (!strcmp("VK_NEWLINE", vkeystr))
      return VK_NEWLINE;
   else if (!strcmp("VK_SHIFT", vkeystr))
      return VK_SHIFT;
   else if (!strcmp("VK_CTRL", vkeystr))
      return VK_CTRL;
   else if (!strcmp("VK_ALT", vkeystr))
      return VK_ALT;
   else if (!strcmp("VK_ALTGRAF", vkeystr))
      return VK_ALTGRAF;
   else if (!strcmp("VK_PAUSE", vkeystr))
      return VK_PAUSE;
   else if (!strcmp("VK_CAPSLOCK", vkeystr))
      return VK_CAPSLOCK;
   else if (!strcmp("VK_ESC", vkeystr))
      return VK_ESC;
   else if (!strcmp("VK_SPACE", vkeystr))
      return VK_SPACE;
   else if (!strcmp("VK_PAGEUP", vkeystr))
      return VK_PAGEUP;
   else if (!strcmp("VK_PAGEDOWN", vkeystr))
      return VK_PAGEDOWN;
   else if (!strcmp("VK_END", vkeystr))
      return VK_END;
   else if (!strcmp("VK_HOME", vkeystr))
      return VK_HOME;
   else if (!strcmp("VK_LEFT", vkeystr))
      return VK_LEFT;
   else if (!strcmp("VK_UP", vkeystr))
      return VK_UP;
   else if (!strcmp("VK_RIGHT", vkeystr))
      return VK_RIGHT;
   else if (!strcmp("VK_DOWN", vkeystr))
      return VK_DOWN;
   else if (!strcmp("VK_PRINTSCRN", vkeystr))
      return VK_PRINTSCRN;
   else if (!strcmp("VK_INSERT", vkeystr))
      return VK_INSERT;
   else if (!strcmp("VK_DELETE", vkeystr))
      return VK_DELETE;
   else if (!strcmp("VK_SCRLLOCK", vkeystr))
      return VK_SCRLLOCK;
   else if (!strcmp("VK_NUMLOCK", vkeystr))
      return VK_NUMLOCK;
   else if (!strcmp("VK_ENTER", vkeystr))
      return VK_ENTER;
   else if (!strcmp("VK_SYSRQ", vkeystr))
      return VK_SYSRQ;
   else if (!strcmp("VK_F1", vkeystr))
      return VK_F1;
   else if (!strcmp("VK_F2", vkeystr))
      return VK_F2;
   else if (!strcmp("VK_F3", vkeystr))
      return VK_F3;
   else if (!strcmp("VK_F4", vkeystr))
      return VK_F4;
   else if (!strcmp("VK_F5", vkeystr))
      return VK_F5;
   else if (!strcmp("VK_F6", vkeystr))
      return VK_F6;
   else if (!strcmp("VK_F7", vkeystr))
      return VK_F7;
   else if (!strcmp("VK_F8", vkeystr))
      return VK_F8;
   else if (!strcmp("VK_F9", vkeystr))
      return VK_F9;
   else if (!strcmp("VK_F10", vkeystr))
      return VK_F10;
   else if (!strcmp("VK_F11", vkeystr))
      return VK_F11;
   else if (!strcmp("VK_F12", vkeystr))
      return VK_F12;
   else if (!strcmp("VK_F13", vkeystr))
      return VK_F13;
   else if (!strcmp("VK_F14", vkeystr))
      return VK_F14;
   else if (!strcmp("VK_F15", vkeystr))
      return VK_F15;
   else if (!strcmp("VK_F16", vkeystr))
      return VK_F16;
   else if (!strcmp("VK_F17", vkeystr))
      return VK_F17;
   else if (!strcmp("VK_F18", vkeystr))
      return VK_F18;
   else if (!strcmp("VK_F19", vkeystr))
      return VK_F19;
   else if (!strcmp("VK_F20", vkeystr))
      return VK_F20;
   else if (!strcmp("VK_F21", vkeystr))
      return VK_F21;
   else if (!strcmp("VK_F22", vkeystr))
      return VK_F22;
   else if (!strcmp("VK_F23", vkeystr))
      return VK_F23;
   else if (!strcmp("VK_F24", vkeystr))
      return VK_F24;
   else if (!strcmp("VK_MENU", vkeystr))
      return VK_MENU;
   else if (!strcmp("VK_COMPOSE", vkeystr))
      return VK_COMPOSE;
   else
      return VK_LEAVEALONE;
}



/*
** KEYMAP_INIT
**
** This routine is reference in DLLINIT.ASM and is called the first time
** the DLL is used.  We'll use this opportunity to read in the scancode
** mappings file.
*/
int FAR PASCAL KEYMAP_INIT(void)
{
   FILE     *fmap;
   CHAR     buff[133];
   CHAR     *cp;
   USHORT   usIn, usOut;
   int      pos;
   CHAR     achVKey[33];
   CHAR     *fname;
   CHAR     str1[10], str2[10];

   /*
   ** Init our mapping arrays of scancodes and virtual keys.
   */
   for (pos=0; pos<256; pos++)
   {
      map_sc[pos] = (UCHAR)pos;
      map_vk[pos] = (UCHAR)VK_LEAVEALONE;
   }

   /*
   ** Open the mappings file.
   */
   DosScanEnv("PMKEYMAP", (PSZ FAR *)&fname);
   if (!fname)
      fname = getenv("PMKEYMAP");
   if (!fname)
      fname = "PMKEYMAP.INI";
   fmap = fopen(fname, "r");

   /*
   ** If we found one, we can parse it...
   */
   if (fmap)
   {
      /*
      ** While we have records to read...
      */
      while(!feof(fmap))
      {
         /*
         ** Get a record...
         */
         fgets(buff, sizeof(buff), fmap);

         /*
         ** See if it's a comment line, if so skip it, otherwise,
         ** continue parsing.
         */
         pos = strspn(buff, " \t\n\r");
         cp = buff + pos;

         if(pos < strlen(buff) && *cp != ';' && *cp != '!')
         {
            /*
            ** Try to get the various pieces...
            */
            *achVKey = '\0';
            if (sscanf(buff, "%s : %s : %s", str1, str2, achVKey))
            {
               /*
               ** ...and store them in our tables.
               */
               usIn = get_hex_dec_oct(str1);
               usOut = get_hex_dec_oct(str2);
               map_sc[usIn] = (UCHAR)usOut;
               if (*achVKey)
                  map_vk[usIn] = TranslateVKey(achVKey);
            }
         }
      }

      /*
      ** Done with the mapping file, close it.
      */
      fclose(fmap);
   }

   /*
   ** Done with DLL init.
   */
   return 1;
}



/*
** KmpComposer
**
** Support routine stolen from SETHOST code.  Given two keys generates
** the composed character.
*/
unsigned int APIENTRY KmpComposer(int key1, int key2)
{ 
	int number_of_compose_sequences,i;
	char *p;

	key1 &= 0x7F;
	key2 &= 0x7F;

	if ( (key1 < ' ') || (key2 < ' ') )
		return 0;

	p = compose_ptr[key1-' '];
	number_of_compose_sequences = *p;
	if (number_of_compose_sequences == 0)
      return 0;

	for (i=1; i<number_of_compose_sequences*2+1; i+=2) 
		if (key2  == *(p+i))
		{
			return(*(p+i+1));
		}

	return 0;
}

/*
** KmpInputHook
**
** Actual hook that will be applied to the system queue.  Takes a 
** look at each and every keystroke and performs further processing
** of interesting ones.  
*/
BOOL APIENTRY KmpInputHook(HAB hab, PQMSG pqmsg, USHORT fs)
{
   /*
   ** Static variables to keep track of composing sequences.
   */
   static int    iComposing=0;
   static USHORT achComposeChars[2];

   USHORT        fsKeyFlags;
   USHORT        usChr;
   USHORT        usVKey;
   UCHAR         uchRepeat;
   UCHAR         uchScanCode; 

   /*
   ** First of all, only look at WM_CHAR messages...
   */
   if (pqmsg->msg == WM_CHAR)
   {
      /*
      ** Extract the fields.
      */
      fsKeyFlags = (USHORT) SHORT1FROMMP(pqmsg->mp1);
      uchRepeat = (UCHAR) CHAR3FROMMP(pqmsg->mp1);   
      uchScanCode = (UCHAR) CHAR4FROMMP(pqmsg->mp1); 
      usChr = (USHORT) SHORT1FROMMP(pqmsg->mp2);     
      usVKey = (USHORT) SHORT2FROMMP(pqmsg->mp2);    

      /*
      ** If we're in the middle of a compose key sequence store this
      ** key or finish the sequence and push the resulting character
      ** into the queue.
      */
      if (iComposing)
      {
         /*
         ** Only watch for the key coming up...
         */
         if (fsKeyFlags&KC_KEYUP && usChr&0x00FF)
         {
            /*
            ** Store it and decrement the number of compose characters
            ** still pending.
            */
            achComposeChars[2-iComposing--] = usChr & 0x00FF;

            /*
            ** If we're still waiting for more characters, swallow this
            ** one and end here.
            */
            if (iComposing)
               return FALSE;  

            /*
            ** Otherwise, get the resulting character.
            */
            else
            {
               usChr = KmpComposer(achComposeChars[0], achComposeChars[1]) & 0x00FF;
               if (!usChr)
               {
                  WinSendMsg(hwndCompose, WM_USER+1, 0L, 0L);
                  return FALSE;
               }
               else
               {
                  usChr = iso2cp(usChr);
                  pqmsg->mp2 = MPFROM2SHORT(usChr, 0);    // No scancode!
                  pqmsg->mp1 = MPFROM2SHORT(KC_CHAR, 1);  // only KC_CHAR, rpt=1
                  return FALSE;  // Pass it on.
               }
            }
         }

         /*
         ** If we're still composing, but this isn't a keyup, swallow it.
         */
         else
            return FALSE;
      }

      /*
      ** Watch for a compose key getting pressed.  At one time, we were
      ** just returning TRUE (swallowing the characters) until the compose
      ** sequence had been finished.  We had problems with things like
      ** Microsoft Excel which might've had a hook on their own message
      ** queue and would get screwed up if grabbed characters from the
      ** queue.  So now, we switch the input focus to a less sensitive
      ** window until the compose sequence is finished.  That less
      ** sensitive window (PMKEYMAP) will then post the resulting character
      ** to the window that originally had the focus and will also set
      ** the focus back to where it's supposed to be.
      */
      if (map_vk[uchScanCode] == VK_COMPOSE)
      {
         if (fsKeyFlags&KC_KEYUP)
         {
            iComposing = 2;
            if (hwndCompose) 
               WinSendMsg(hwndCompose, WM_USER, pqmsg->hwnd, 0L);
         }
         return FALSE;
      }

      /*
      ** If we're supposed to assign a new virtual key then do so.
      */
      if (map_vk[uchScanCode] != (UCHAR)VK_LEAVEALONE)
      {
         usVKey = map_vk[uchScanCode];
         fsKeyFlags |= KC_VIRTUALKEY;
      }

      /*
      ** Map the character.
      */
      uchScanCode = map_sc[uchScanCode];

      /*
      ** Patch the message with the new character information.
      */
      pqmsg->mp1 = MPFROM2SHORT(fsKeyFlags, ((USHORT)uchScanCode)<<8|uchRepeat);
      pqmsg->mp2 = MPFROM2SHORT(usChr, usVKey);
   }
      
   return FALSE;   // Pass it on down the input chain...
}



/*
** KmpSetComposerWnd
**
** Let the input hook know where it's supposed to switch the focus to
** when a compose sequence is initiated.
*/
VOID APIENTRY KmpSetComposerWnd(HWND hwnd)
{
   hwndCompose = hwnd;
}



/*
** KmpSetCP
**
** Let an application set the codepage that it wants to composer use
** in translating ISO-LATIN-1.  The use of this function should be
** tied to WM_SETFOCUS messages.  The old codepage is returned and
** when focus is lost, the old codepage should be restored.  This
** is currently all up to the calling application.
*/
int APIENTRY KmpSetCP(int cp)
{
   int iCurr = iCodePage;

   iCodePage = cp;
   return iCurr;
}

