#include <dos.h>
#include <stdio.h>

void
listdrives()
   {
   union REGS r;
   struct SREGS sr;
   
   int lastdrive;                         // based on A=1
   short default_drive;                   // based on A=1
   int i;
   
   int drive_type;
   volatile char  *remote;                // ptr to string "LOCAL" or "REMOTE"
   
   long total_space,
        free_space;
   
   char logical;                          // used to * logical drives
   int logical_count=0;                   // count of non-physical devices

   struct bios_block
      {
      char special;
      char devtype;
      int removable;
      int cylinders;
      char media;
      char BPB_field[31];
      } bios_block;
   
   char *ptr;                 // used to send location of bios_block
                              // to DOS. 
   
   static char *drtypes[] = {     // this list current as of DOS 4.01
                             "360K",
                             "1.2MB",
                             "720K",
                             "SD8",
                             "DD8",
                             "Fixed",
                             "Tape",
                             "1.44MB"
                             };
   
   r.h.ah = 0x19;                 // get current drive
   intdos (&r,&r);
   default_drive = r.h.al;
   
   r.h.ah = 0x0E;
   r.h.dl = default_drive;       // set default drive function returns
                                 // lastdrive
   intdos (&r,&r);
   
   lastdrive = r.h.al;
 
  // printf("LASTDRIVE is %c:\n",lastdrive+64);
  // printf("Current drive is %c:\n",default_drive+65);
   
   printf("\nDrive Information\n\n");
   printf("Drive    Type      Local/Remote   Total Space (K)      Free Space (K)\n");
   printf("-----    ------    ------------   ---------------      --------------\n");
   
   for (i=1;i<lastdrive;i++)
      {
      
      logical = ' ';
      
      r.h.ah = 0x44;
      r.h.al = 0x0D;
      r.h.bl = i;
      r.h.ch = 0x08;
      r.h.cl = 0x60;
      ptr = (char *)&bios_block;
      segread(&sr);                   // get the segment registers set
      r.x.dx = FP_OFF(ptr);           // and set the offset to bios_block
      
      intdosx(&r,&r,&sr);
      
      total_space = 0;
      free_space = 0;
      remote = "Local";
      if (bios_block.devtype == 5)
         {
         
         r.h.ah = 0x44;   // determine if local or remote
         r.h.al = 0x09;
         r.h.bl = i;
         intdos(&r,&r);
         if (r.x.cflag)   // carry flag indicates invalid drive
            continue;
         if (r.x.dx & 0x0008)
            remote="Remote";
         
            r.h.ah = 0x36;       // get free space on drive
            r.h.dl = i;
            intdos(&r,&r);
         
            total_space = ((long)r.x.ax*(long)r.x.cx*(long)r.x.dx)/1024;
            free_space  = ((long)r.x.ax*(long)r.x.cx*(long)r.x.bx)/1024;
         
          }
         else
          {
          r.h.ah = 0x44;
          r.h.al = 0x0E;
          r.x.bx = i;
          intdos(&r,&r);
          if (r.h.al && r.h.al != i) 
             {
             logical='*';
             logical_count++;
             }
          }
          
      if (bios_block.devtype==5)
         printf(" %c:     %c%-6s    %-12s          %8ld            %8ld\n",
            i+64,logical,drtypes[bios_block.devtype],remote,total_space,free_space);
        else
         printf(" %c:     %c%-6s    %-12s\n",
               i+64,logical,drtypes[bios_block.devtype],remote);
               
      }
      
      if (logical_count)
         printf("\n*  Indicates LOGICAL name for another PHYSICAL device.\n");
  }

/* --------------------
From MS-DOS encyclopedia:

        
        Special-functions field: offset 00H, length 1 byte
        
        Bit         Value       Meaning
        
        0           0           Returns default BIOS parameter block (BPB)
                                for the device.
                    1           Returns BPB that the Build BPB device
                                driver call would return.
        1-7         0           Reserved (must be zero).

        
        Device type field: offset 01H, length 1 byte
        
        Value       Meaning
        
        00          320/360 KB 5.25-inch disk
        01          1.2 MB 5.25-inch disk
        02          720 KB 3.5-inch disk
        03          Single-density 8-inch disk
        04          Double-density 8-inch disk
        05          Fixed disk
        06          Tape drive
        07          Other type of block device

        
        Device attributes field: offset 02H, length 1 word
        
        Bit         Value       Meaning
        
        0           0           Removable storage medium
                    1           Nonremovable storage medium
        1           0           Door lock not supported
                    1           Door lock supported
        2-15        0           Reserved

        
        Number of cylinders field: offset 04H, length 1 word
        
        Meaning: Maximum number of cylinders supported; set by device
                 driver

        
        Media type field: offset 06H, length 1 byte
        

        Value               Meaning
        
        00H (default)       1.2 MB 5.25-inch disk
        01H                 320/360 KB 5.25-inch disk

        
        Device BPB field: offset 07H, length 31 bytes
        
        Meaning: See Programmer's Note below.
        
        If bit 0 = 0 in special-functions field, this field contains the
        new default BPB for the device.

        If bit 0 = 1 in special-functions field, BPB in this field is
        returned by the device driver in response to subsequent Build BPB
        requests.

        
        Track layout field: offset 26H
        
        Unused
        

       The device BPB field is a 31-byte data structure. Information
        contained in the device BPB field describes the current disk and
        disk control areas. The device BPB field is formatted as follows:

        Byte          Meaning
        
        00-01H        Number of bytes per sector
        02H           Number of sectors per allocation unit
        03-04H        Number of sectors reserved, beginning at sector 0
        05H           Number of file allocation tables (FATs)
        06-07H        Maximum number of root-directory entries
        08-09H        Total number of sectors
        0AH           Media descriptor
        0B-0CH        Number of sectors per FAT
        0D-0EH        Number of sectors per track
        0F-10H        Number of heads
        11-14H        Number of hidden sectors
        15-1FH        Reserved
------------------------------------------------------------------------*/
