/*
** PMKEYMAP.C
** 
** Copyright (c) 1989 by Digital Equipment Corporation
**                    All rights reserved.
**
** This module contains an application to set an input hook
** on the system message queue.  This particular hook is for
** translating scancodes and for supporting VMS-style compose
** sequences.  This has an immediate appliction as support for
** the LK250 in DEC mode, but isn't necessarily tied to that
** keyboard.
*/



/*
** Includes
*/
#define INCL_PM
#define INCL_DOS
#define INCL_DEV
#include <os2.h>
#include "keymap.h"
#include "pmkeymap.h"

/*
** Global variables - Too lazy to remove the use of these...
*/
HAB              hab;
HMQ              hmq;
HWND             hwndSave;



/*
** SetMappingState
**
** Support routine to hook/unhook the system queue and keep the
** menu up to date.
*/
VOID SetMappingState(HWND hwnd, USHORT mid)
{
   USHORT              usResult;
   BOOL                bResult;
   char                buff[40];

   static USHORT       currstate=0;
   static HMODULE      hmod=NULL;
   static HWND         hwndMenu=NULL;

   if (!hwndMenu)
      hwndMenu = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT, FALSE),
                                 FID_MENU) ;

   if (mid==currstate)
      return;

   if (mid==MID_KMP_ON)
   {
      usResult = DosLoadModule(buff, sizeof(buff), "KEYMAP", &hmod);
      bResult = WinSetHook(hab, NULL, HK_INPUT, (PFN)KmpInputHook, hmod);
      KmpSetComposerWnd(hwnd);
   }
   else
   {
      bResult = WinReleaseHook(hab, NULL, HK_INPUT, (PFN)KmpInputHook, hmod);
      usResult = DosFreeModule(hmod);
   }

   if (currstate)
      WinSendMsg(hwndMenu, MM_SETITEMATTR, 
                 MPFROM2SHORT(currstate, TRUE), 
                 MPFROM2SHORT(MIA_CHECKED, 0));

   currstate = mid;

   WinSendMsg(hwndMenu, MM_SETITEMATTR, 
              MPFROM2SHORT(currstate, TRUE), 
              MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED));
   return;
}



/*
** main
**
** The main procedure to create the window and dispatch messages to it.
*/
void main(void)
{
   HWND       hwndFrame, hwndClient;
   QMSG       qmsg;
   ULONG      flFrameFlags = FCF_TITLEBAR      | FCF_SYSMENU  |
                             FCF_SIZEBORDER    | FCF_MINMAX   |
                             FCF_SHELLPOSITION | FCF_TASKLIST |
                             FCF_ICON          | FCF_MENU;

   /*
   ** Typical PM app startup.  Create anchor block, message queue and
   ** window.
   */
   hab = WinInitialize(0);
   hmq = WinCreateMsgQueue(hab, 0);

   WinRegisterClass(hab, "PMKEYMAP", KeymapWndProc, CS_SIZEREDRAW, 0);

   hwndFrame = WinCreateStdWindow (
                    HWND_DESKTOP,       // Parent window handle
                    WS_VISIBLE,         // Style of frame window
                    &flFrameFlags,      // Pointer to control data
                    "PMKEYMAP",         // Client window class name
                    NULL,               // Title bar text
                    0L,                 // Style of client window
                    NULL,               // Module handle for resources
                    ID_RESOURCE,        // ID of resources
                    &hwndClient) ;      // Pointer to client window handle

   /*
   ** Now dispatch messages to the window.
   */
   while (WinGetMsg(hab, &qmsg, NULL, 0, 0))
        WinDispatchMsg(hab, &qmsg);

   /*
   ** When WinGetMsg fails, we're done.  We can kill things and exit.
   */
   WinDestroyWindow(hwndFrame);
   WinDestroyMsgQueue(hmq);
   WinTerminate(hab);

   return;
}



/*
** KeymapWndProc
**
** This is the window procedure for our PMKEYMAP application.  It doesn't
** do too much with it's client area.  It will eventually support a
** menu that will allow the user to Enable/Disable the hook.
*/
MRESULT EXPENTRY KeymapWndProc(HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
{
   /*
   ** Usual window handling:  switch of the message, ignoring most.
   */
   switch(msg)
   {
   /*
   ** WM_CREATE
   **
   ** Load the DLL containing the system hook and set it up.
   */
   case WM_CREATE:
      SetMappingState(hwnd, MID_KMP_ON);
      break;

   /*
   ** WM_DESTROY
   **
   ** Remove the hook and free the DLL.
   */
   case WM_DESTROY:
      SetMappingState(hwnd, MID_KMP_OFF);
      break;

   /*
   ** WM_ERASEBACKGROUND
   **
   ** Return TRUE so that we have some kind of window background.  We might
   ** get rid of this if we ever decide to paint the client area.
   */
   case WM_ERASEBACKGROUND:
      return TRUE;

   /*
   ** WM_USER message indicates a window that has started a compose
   ** sequence.  We should grab the focus and accept the input.
   */
   case WM_USER:
      hwndSave = mp1;
      WinFocusChange(HWND_DESKTOP, hwnd,
                     FC_NOLOSEACTIVE|FC_NOLOSEFOCUS|FC_NOLOSESELECTION|
                     FC_NOSETACTIVE|FC_NOSETFOCUS|FC_NOSETSELECTION);
      break;

   /*
   ** WM_USER + 1
   **
   ** Abort.
   */
   case WM_USER+1:
      WinSetFocus(HWND_DESKTOP, hwndSave);
      hwndSave = NULL;
      break;

   /*
   ** WM_CHAR
   **
   ** post the composed character.
   */
   case WM_CHAR:
      if (hwndSave && !CHARMSG(&msg)->scancode && 
          CHARMSG(&msg)->fs==KC_CHAR && CHARMSG(&msg)->chr)
      {
         WinSetFocus(HWND_DESKTOP, hwndSave);
         WinPostMsg(hwndSave, WM_CHAR, mp1, mp2);
         hwndSave = NULL;
      }
      break;

   /*
   ** WM_COMMAND
   **
   ** Handle menu activity.
   */
   case WM_COMMAND:
      switch(SHORT1FROMMP(mp1))
      {
      case MID_ABOUT:
         WinDlgBox(hwnd, hwnd, AboutDlgProc, NULL, DID_ABOUT, NULL);
         break;

      case MID_KMP_ON:
         SetMappingState(hwnd, MID_KMP_ON);
         break;

      case MID_KMP_OFF:
         SetMappingState(hwnd, MID_KMP_OFF);
         break;

      default:
         return WinDefWindowProc(hwnd, msg, mp1, mp2);
      }
      break;

   /*
   ** default
   **
   ** Let the Presentation Manager deal with it.
   */
   default:
      return WinDefWindowProc(hwnd, msg, mp1, mp2);
   }

   /*
   ** Return value.
   */
   return 0L;
}



MRESULT EXPENTRY AboutDlgProc(HWND hwnd, USHORT msg, MPARAM mp1, MPARAM mp2)
{
   switch(msg)
   {
   case WM_COMMAND:
      WinDismissDlg(hwnd, TRUE);
      break;

   default:
      return WinDefDlgProc(hwnd, msg, mp1, mp2);
   }

   return 0L;
}
